#!/usr/bin/python
# -*- coding: UTF-8 -*-
# 08.05.2016
"""
Wird aus dem Hauptmenü aufgerufen und ist für das Ändern von Einträgen zuständig.

Parameter: conditon = Filterbedingung, label = Teil der Beschriftung des Fensters

Es wird zunächst ein Fenster geöffnet, in dem die Auswahl der Datensätze eingeschränkt werden kann.
Danach wird eine Liste aller Einträge aufgebaut (TreeView).
Innerhalb der Liste kann mit einem Doppelclick der Linken Maustaste oder Rechte Maustaste eine Zeile/Buchung zum
Ändern selektiert werden.
Die iid der Zeile ist so gewählt, dass sie mit dem Primary key des zugehörigen Tabelleneintrags
übereinstimmt.
Somit kann zum Editieren der Namen der Tabelle in Verbindung mit dem Primary Key genau ein Datensatz
eingelesen, angezeigt und ggf. verändert werden.
"""

import tkinter as tk
from tkinter import ttk
from tkinter import messagebox
import database as db
from buchung import Buchung
import locale as lc
lc.setlocale(lc.LC_ALL, "")


class Change(tk.Toplevel):
    def __init__(self, parent, condition, label):
        tk.Toplevel.__init__(self)

        self.resizable(width=False, height=False)
        self.bind("<Escape>", self.cancel)
        self.protocol("WM_DELETE_WNDOW", self.cancel)
        self.title("{} ändern, Filter: {}".format(db.label, label))
        self.geometry("+{}+{}".format(parent.winfo_rootx()+50, parent.winfo_rooty()+50))

        Treeview(self, condition)

        self.focus_set()           # focus auf das Fenster - hier nicht
        self.grab_set()            # alle Events werden nur nach w durchgereicht,
        #                            das drunterliegende Fenster bleibt inaktiv, wird aber aktualisiert
        self.transient(parent)     # das Fenster bleibt oberhalb von Top
        self.wait_window(self)     # warten bis zum Beenden des Fensters

    def cancel(self, e=None):
        self.destroy()


class Treeview(tk.Frame):
    def __init__(self, parent, condition):
        tk.Frame.__init__(self, parent)
        self.grid(column=0, row=0, padx=5, pady=5, sticky="nsew")
        self.tree = ttk.Treeview(self, height=20, columns=('datum', 'text1', 'text2', "brutto", 'netto', 'ust'),
                                 selectmode='browse')
        scrollbar_y = ttk.Scrollbar(self, orient="vertical", command=self.tree.yview)
        # scrollbar_x = ttk.Scrollbar(self, orient="horizontal", command=self.tree.xview)
        # self.tree.configure(yscroll=scrollbar_y.set, xscroll=scrollbar_x.set)
        self.tree.configure(yscroll=scrollbar_y.set)
        self.tree.focus_set()

        # Spalten generieren
        spalten = (('Konto', 175, 'center'),
                   ('Datum', 70, 'center'),
                   ('Textzeile 1', 190, 'center'),
                   ('Textzeile 2', 190, 'center'),
                   ('Brutto', 100, 'center'),
                   ('Netto', 100, 'center'),
                   ('Ust.', 100, 'center'))
        for x, s in enumerate(spalten):
            nr = '#' + str(x)
            self.tree.heading(nr, text=s[0], anchor=s[2])
            self.tree.column(nr, width=s[1])

        self.tree.grid(row=0, column=0, padx=5, pady=5, sticky='nsew')
        scrollbar_y.grid(column=1, row=0, sticky='ns')
        # scrollbar_x.grid(column=0, row=1, sticky='ew')

        self.populate(condition)

    def populate(self, condition):
        """
        Füllt die Tabelle mit Konten und den untergeordneten Buchungseinträgen
        :param condition: Filterbedingung für die Datenbank
        """

        # Die Einträge werden nach Konto und Datum sortiert erwartet, daher mit Kont0o 0 beginnnen
        konto = 0

        # die gewünschten Datensätze einlesen, der Name der Tabelle steht in db.table
        # hier werden Wurzeleinträge nur für Konten mit bestehenden Einträgen angelegt
        rows = db.get_data(db.table, condition=condition)
        for row in rows:
            # die Tabelle muss sortiert sein nach 'konto, date'
            # dann werden hier Wurzeleinträge für die Konten angelegt
            if row[1] > konto:
                konto = row[1]
                self.populate_root(row[1])

            self.populate_row(row)

        # den Font für den Treeview festlegen
        self.tree.tag_configure("ausgabe", font=("Courier", 9))
        self.tree.tag_configure("konto", font=("Courier", 9))

        # Events bei Mausclick/Taste definieren - Doppelclick Linke Maustaste und Rechte Maustaste
        self.tree.tag_bind('ausgabe', '<Double-1>', self.itemClicked_1)
        self.tree.tag_bind('ausgabe', '<3>', self.itemClicked_1)

    def populate_root(self, konto):
            kontoname = db.get_name(konto)
            # wenn ein Kontofilter besteht (also auch nur 1 Konto sichtbar ist,
            # dann wird dieses (einzige) Konto auch gleich 'aufgeklappt'
            if db.kontofilter:
                menu_open = True
            else:
                menu_open = False
            self.tree.insert('', 'end', iid='K_' + str(konto), text=kontoname, tags='konto', open=menu_open)

    def populate_row(self, row):
            # den Listeneintrag vorbereiten
            datum = str(row[2])
            date = "{}.{}.{}".format(datum[4:], datum[2:4], datum[:2])
            brutto = row[5]/100 + row[6]/100
            brutto = "{:>13}".format(lc.currency(brutto, symbol=False, grouping=True))
            netto = "{:>13}".format(lc.currency(row[5]/100, symbol=False, grouping=True))
            ust = "{:>13}".format(lc.currency(row[6]/100, symbol=False, grouping=True))
            self.tree.insert('K_{}'.format(row[1]), 'end', iid=row[0], text=row[0],
                             values=(date, row[3], row[4], brutto, netto, ust), tags="ausgabe")

    def itemClicked_1(self, e=None):
        """
        Den aktuell selektierten Datensatz bearbeiten
        :param e:
        """
        iid = self.tree.identify_row(e.y)
        Buchung(self, key=iid, callback=self.callback, )

    def delete(self, rec):
        # den Datensatz in der Datenbank löschen und dann das Konto neu einlesen
        db.delete_rec(db.table, rec)
        self.re_populate_konto(rec, int(self.tree.parent(rec)[2:]))

    def callback(self, rec):
        """
        Wird beim Ändern benutzt, um die Änderung in den Treeview einzupflegen
        Kompliziert wird es, wenn das Konto oder das Datum des Eintrages geändert wird:
        - bei geändertem Konto muss der Eintrag umgehängt werden in anderen Wurzeleintrag
        - bei Datumänderung müsste der Eintrag innerhalb des Wurzeleintrages neu eingeordnet werden
        """
        # wenn nur 1 Feld übergeben wird, dann handelt es sich um den Key für einen zu löschenden Datensatz
        if len(rec) == 1:
            self.delete(rec[0])
            return

        # die Id = Primary Key des Datensatzes bleibt unverändert, darüber kann man ihn eindeutig identifizieren
        db_id = rec[0]
        new_konto = rec[1]
        # die iid des Wurzeleintrages besteht aus 'K_' + Konto
        # den Wert hinter 'K_' isolieren, das ist das alte Konto
        oldkonto = int(self.tree.parent(rec[0])[2:])

        # die alten Werte (vor der Änderung) aus dem Treeview auslesen
        oldvalues = self.tree.item(db_id, 'values')
        olddate = oldvalues[0]

        # einen einzelnen Datensatz korrigieren
        # Das Datenformat von rec[]: key, konto, date, text1, text2, brutto, netto, ust
        newdate = str(rec[2])
        # newdate = "{}.{}.{}".format(datum[4:], datum[2:4], datum[:2])

        # wurde die Kontenzuordnung geändert ?
        if not oldkonto == new_konto:
            # print("Konto verändert")
            # konto = rec[1]
            # den aktuellen Eintrag im Treeview löschen (nicht in der Datenbank)
            self.tree.delete(db_id)
            # Der Eintrag muss nun in ein anderes Konto eingefügt werden
            # das ist aber sinnlos, wenn ein Kontofilter besteht, dann wird nur dieses eine Konto
            # angezeigt.
            # Daher kann in diesem Fall beendet werden
            if db.kontofilter:
                messagebox.showinfo(
                    "Buchung ändern", "Der Eintrag wurde in ein ausgeblendetes Konto verschoben und" +
                    " wird daher aktuell nicht mehr angezeigt !")
                return

            # es besteht kein Filter für die Konten, daher muss der Eintrag im neuen Konto an der zeitlich richtigen
            # Position eingefügt werden
            # dazu werden zuerst alle Elemente des neuen Kontos gelöscht
            # und anschließend werden alle Einträge neu aus der Datenbank eingelesen - einschließlich des
            # verschobenen Eintrages
            parent = 'K_{}'.format(new_konto)
            # falls das Konto nicht angezeigt wird, dann einen Eintrag anlegen
            if not self.tree.exists(parent):
                # print("Eintrag anlegen")
                self.populate_root(new_konto)
            # wenn der Eintrag bereits besteht, dann alle Unterelemente löschen
            else:
                children = self.tree.get_children(parent)
                for child in children:
                    self.tree.delete(child)
            # Bevor das Konto aus der Datenbank gelesen wird, abhängig vom zur Zeit aktuellen Filter
            # einen neuen Filter anlegen
            if db.timefilter:
                filter = "WHERE {} and konto = {}".format(db.timefilter, new_konto)
            else:
                filter = "WHERE konto = {}".format(new_konto)
            # alle Daten des neuen Kontos holen
            rows = db.get_data(db.table, condition=filter)
            # und einfügen
            for row in rows:
                self.populate_row(row)
            # sollte der Eintrag nicht existieren, dann ist er aus dem Zeitfilter gefallen
            if not self.tree.exists(db_id):
                messagebox.showinfo(
                    "Buchung ändern", "Der Eintrag wurde aus dem gewählten Zeitfenster verschoben und" +
                    " wird daher nicht mehr angezeigt !")
            else:
                self.tree.see(rec[0])
                self.tree.selection_set(rec[0])

        # wenn das Datum verändert wurde, dann muss der Eintrag lediglich innerhalb seines Parents verschoben werden.
        # Dazu werden alle Untereinträge gelöscht und neu aus der Datenbank eingelesen.
        elif not newdate == olddate:
            # print("Datum verändert")
            # alle Untereinträge des Kontos löschen und neu einlesen
            self.re_populate_konto(db_id, oldkonto)
            return

        else:
            # Weder das Konto noch das Datum wurden geändert, es reicht aus, die Zeile neu zu schreiben
            # print(rec[5], rec[6], rec[7])
            self.tree.item(
                rec[0], values=(olddate, rec[3], rec[4],
                "{:>13}".format(lc.currency(rec[5], symbol=False, grouping=True)),
                "{:>13}".format(lc.currency(rec[6], symbol=False, grouping=True)),
                "{:>13}".format(lc.currency(rec[7], symbol=False, grouping=True))))
            self.tree.see(rec[0])

    def re_populate_konto(self, id, konto):
        """ Die gesamte Liste eines Kontos neu aufbauen """
        parent = self.tree.parent(id)
        children = self.tree.get_children(parent)
        for child in children:
            self.tree.delete(child)
        if db.timefilter:
            filter = "WHERE {} and konto = {}".format(db.timefilter, konto)
        else:
            filter = "WHERE konto = {}".format(konto)

        rows = db.get_data(db.table, condition=filter)
        for row in rows:
            self.populate_row(row)

        # möglicherweise ist der Eintrag wegen eines Filters nicht mehr sichtbar
        if not self.tree.exists(id):
            messagebox.showinfo(
                "Buchung ändern", "Der Eintrag wurde gelöscht oder aus dem gewählten Zeitfenster verschoben und" +
                " wird aktuell nicht nehr angezeigt !")
        else:
            self.tree.see(id)
            self.tree.selection_set(id)


if __name__ == "__main__":
    top = tk.Tk()
    db.open_db()
    db.init_konten()
    db.define_table("ausgaben")

    top.title("Treeview - Test")
    Change(top, condition="", label="Test")
    db.close_db()